@@ -12,10 +12,7 @@
 class CEPlugin : public KTextEditor::Plugin
 {
 public:
-    /**
-     * Plugin constructor.
-     */
-    explicit CEPlugin(QObject *parent = nullptr, const QVariantList & = QVariantList());
+    explicit CEPlugin(QObject *parent);
 
     ~CEPlugin() override;
 
