@@ -4,8 +4,7 @@
  *
  *  SPDX-License-Identifier: LGPL-2.0-or-later
  */
-#ifndef KTEXTEDITOR_KATE_EXTERNALTOOLS_COMMAND_H
-#define KTEXTEDITOR_KATE_EXTERNALTOOLS_COMMAND_H
+#pragma once
 
 #include <KTextEditor/Command>
 
@@ -32,6 +31,4 @@ private:
     KateExternalToolsPlugin *m_plugin;
 };
 
-#endif // KTEXTEDITOR_KATE_EXTERNALTOOLS_COMMAND_H
-
 // kate: space-indent on; indent-width 4; replace-tabs on;
