@@ -4,6 +4,7 @@
     SPDX-License-Identifier: LGPL-2.0-or-later
 */
 #include "client.h"
+#include <QFileInfo>
 #include <QJsonArray>
 #include <QJsonDocument>
 #include <limits>
@@ -17,22 +18,72 @@ namespace dap
 {
 constexpr int MAX_HEADER_SIZE = 1 << 16;
 
-Client::Client(const settings::ProtocolSettings &protocolSettings, Bus *bus, QObject *parent)
+struct ClientMessageContext : public MessageContext {
+    Utils::PathMappingPtr pathMap;
+
+    ClientMessageContext(Utils::PathMappingPtr pm)
+        : pathMap(pm)
+    {
+    }
+
+    QString toRemote(const QUrl &url) override
+    {
+        if (!pathMap)
+            return url.toLocalFile();
+
+        auto result = Utils::mapPath(*pathMap, url, true).toLocalFile();
+        // if not able to map (e.g. src not present in remote env), then pass as-is
+        // e.g. gdb might be able to find some heuristic match in symbol info
+        return result.isEmpty() ? url.toLocalFile() : result;
+    }
+
+    virtual QUrl toLocal(const QString &path) override
+    {
+        auto url = QUrl::fromLocalFile(path);
+        // pass along relative paths; can not be reasonably mapped
+        // is a matter of search paths to resolve those
+        if (!pathMap || QFileInfo(path).isRelative())
+            return url;
+
+        return Utils::mapPath(*pathMap, url, false);
+    }
+};
+
+class Client::MessageParser
+{
+public:
+    QByteArray m_buffer;
+
+    void push(QByteArray d)
+    {
+        m_buffer.append(d);
+    }
+    QByteArray read();
+    std::optional<Client::HeaderInfo> readHeader();
+};
+
+Client::Client(const settings::ProtocolSettings &protocolSettings, Bus *bus, Utils::PathMappingPtr pm, QObject *parent)
     : QObject(parent)
     , m_bus(bus)
     , m_managedBus(false)
     , m_protocol(protocolSettings)
     , m_launchCommand(extractCommand(protocolSettings.launchRequest))
+    , m_msgParser(new MessageParser())
 {
+    m_msgContext = std::make_unique<ClientMessageContext>(pm);
     bind();
 }
 
-Client::Client(const settings::ClientSettings &clientSettings, QObject *parent)
+Client::Client(const settings::ClientSettings &clientSettings, Utils::PathMappingPtr pm, QObject *parent)
     : QObject(parent)
     , m_managedBus(true)
     , m_protocol(clientSettings.protocolSettings)
     , m_launchCommand(extractCommand(clientSettings.protocolSettings.launchRequest))
+    , m_msgParser(new MessageParser())
+    , m_outputMsgParser(new MessageParser())
 {
+    m_msgContext = std::make_unique<ClientMessageContext>(pm);
+    m_checkExtraData = pm.get();
     m_bus = createBus(clientSettings.busSettings);
     m_bus->setParent(this);
     bind();
@@ -41,6 +92,8 @@ Client::Client(const settings::ClientSettings &clientSettings, QObject *parent)
 Client::~Client()
 {
     detach();
+    delete m_msgParser;
+    delete m_outputMsgParser;
 }
 
 void Client::bind()
@@ -50,8 +103,7 @@ void Client::bind()
     connect(m_bus, &Bus::closed, this, &Client::finished);
     if (m_protocol.redirectStderr)
         connect(m_bus, &Bus::serverOutput, this, &Client::onServerOutput);
-    if (m_protocol.redirectStdout)
-        connect(m_bus, &Bus::processOutput, this, &Client::onProcessOutput);
+    connect(m_bus, &Bus::processOutput, this, &Client::onProcessOutput);
 }
 
 void Client::detach()
@@ -98,7 +150,7 @@ void Client::processProtocolMessage(const QJsonObject &msg)
     } else if (DAP_REQUEST == type) {
         processReverseRequest(msg);
     } else {
-        qCWarning(DAPCLIENT) << "unknown, empty or unexpected ProtocolMessage::" << DAP_TYPE << " (" << type << ")";
+        qCWarning(DAPCLIENT, "unknown, empty or unexpected ProtocolMessage::%ls (%ls)", qUtf16Printable(DAP_TYPE), qUtf16Printable(type));
     }
 }
 
@@ -120,7 +172,7 @@ void Client::processResponse(const QJsonObject &msg)
     const Response response(msg);
     // check sequence
     if ((response.request_seq < 0) || !m_requests.contains(response.request_seq)) {
-        qCWarning(DAPCLIENT) << "unexpected requested seq in response";
+        qCWarning(DAPCLIENT, "unexpected requested seq in response");
         return;
     }
 
@@ -128,10 +180,10 @@ void Client::processResponse(const QJsonObject &msg)
 
     // check response
     if (response.command != request.command) {
-        qCWarning(DAPCLIENT) << "unexpected command in response: " << response.command << " (expected: " << request.command << ")";
+        qCWarning(DAPCLIENT, "unexpected command in response: %ls (expected: %ls)", qUtf16Printable(response.command), qUtf16Printable(request.command));
     }
     if (response.isCancelled()) {
-        qCWarning(DAPCLIENT) << "request cancelled: " << response.command;
+        qCWarning(DAPCLIENT, "request cancelled: %ls", qUtf16Printable(response.command));
     }
 
     if (!response.success) {
@@ -153,15 +205,15 @@ void Client::processReverseRequest(const QJsonObject &msg)
 void Client::processResponseInitialize(const Response &response, const QJsonValue &)
 {
     if (m_state != State::Initializing) {
-        qCWarning(DAPCLIENT) << "unexpected initialize response";
+        qCWarning(DAPCLIENT, "unexpected initialize response");
         setState(State::None);
         return;
     }
 
     if (!response.success && response.isCancelled()) {
-        qCWarning(DAPCLIENT) << "InitializeResponse error: " << response.message;
+        qCWarning(DAPCLIENT, "InitializeResponse error: %ls", qUtf16Printable(response.message));
         if (response.errorBody) {
-            qCWarning(DAPCLIENT) << "error" << response.errorBody->id << response.errorBody->format;
+            qCWarning(DAPCLIENT, "error %d %ls", response.errorBody->id, qUtf16Printable(response.errorBody->format));
         }
         setState(State::None);
         return;
@@ -181,7 +233,7 @@ void Client::processEvent(const QJsonObject &msg)
 
     if (QStringLiteral("initialized") == event) {
         if ((m_state != State::Initializing)) {
-            qCWarning(DAPCLIENT) << "unexpected initialized event";
+            qCWarning(DAPCLIENT, "unexpected initialized event");
             return;
         }
         setState(State::Initialized);
@@ -191,7 +243,7 @@ void Client::processEvent(const QJsonObject &msg)
         const int exitCode = body[QStringLiteral("exitCode")].toInt(-1);
         Q_EMIT debuggeeExited(exitCode);
     } else if (DAP_OUTPUT == event) {
-        Q_EMIT outputProduced(Output(body));
+        Q_EMIT outputProduced(Output(body, *m_msgContext));
     } else if (QStringLiteral("process") == event) {
         Q_EMIT debuggingProcess(ProcessInfo(body));
     } else if (QStringLiteral("thread") == event) {
@@ -203,9 +255,9 @@ void Client::processEvent(const QJsonObject &msg)
     } else if (QStringLiteral("continued") == event) {
         Q_EMIT debuggeeContinued(ContinuedEvent(body));
     } else if (DAP_BREAKPOINT == event) {
-        Q_EMIT breakpointChanged(BreakpointEvent(body));
+        Q_EMIT breakpointChanged(BreakpointEvent(body, *m_msgContext));
     } else {
-        qCWarning(DAPCLIENT) << "unsupported event: " << event;
+        qCWarning(DAPCLIENT, "unsupported event: %ls", qUtf16Printable(event));
     }
 }
 
@@ -242,7 +294,7 @@ void Client::processResponseStackTrace(const Response &response, const QJsonValu
 {
     const int threadId = request.toObject()[DAP_THREAD_ID].toInt();
     if (response.success) {
-        Q_EMIT stackTrace(threadId, StackTraceInfo(response.body.toObject()));
+        Q_EMIT stackTrace(threadId, StackTraceInfo(response.body.toObject(), *m_msgContext));
     } else {
         Q_EMIT stackTrace(threadId, StackTraceInfo());
     }
@@ -252,7 +304,7 @@ void Client::processResponseScopes(const Response &response, const QJsonValue &r
 {
     const int frameId = request.toObject()[DAP_FRAME_ID].toInt();
     if (response.success) {
-        Q_EMIT scopes(frameId, Scope::parseList(response.body.toObject()[DAP_SCOPES].toArray()));
+        Q_EMIT scopes(frameId, Scope::parseList(response.body.toObject()[DAP_SCOPES].toArray(), *m_msgContext));
     } else {
         Q_EMIT scopes(frameId, QList<Scope>());
     }
@@ -314,7 +366,7 @@ void Client::processResponseDisconnect(const Response &response, const QJsonValu
 void Client::processResponseSource(const Response &response, const QJsonValue &request)
 {
     const auto req = request.toObject();
-    const auto path = req[DAP_SOURCE].toObject()[DAP_PATH].toString();
+    const auto path = m_msgContext->toLocal(req[DAP_SOURCE].toObject()[DAP_PATH].toString());
     const auto reference = req[DAP_SOURCE_REFERENCE].toInt(0);
     if (response.success) {
         Q_EMIT sourceContent(path, reference, SourceContent(response.body.toObject()));
@@ -326,13 +378,13 @@ void Client::processResponseSource(const Response &response, const QJsonValue &r
 
 void Client::processResponseSetBreakpoints(const Response &response, const QJsonValue &request)
 {
-    const auto source = Source(request.toObject()[DAP_SOURCE].toObject());
+    const auto source = Source(request.toObject()[DAP_SOURCE].toObject(), *m_msgContext);
     if (response.success) {
         const auto resp = response.body.toObject();
         if (resp.contains(DAP_BREAKPOINTS)) {
             QList<Breakpoint> breakpoints;
             for (const auto &item : resp[DAP_BREAKPOINTS].toArray()) {
-                breakpoints.append(Breakpoint(item.toObject()));
+                breakpoints.append(Breakpoint(item.toObject(), *m_msgContext));
             }
             Q_EMIT sourceBreakpoints(source.path, source.sourceReference.value_or(0), breakpoints);
         } else {
@@ -360,7 +412,7 @@ void Client::processResponseEvaluate(const Response &response, const QJsonValue
 void Client::processResponseGotoTargets(const Response &response, const QJsonValue &request)
 {
     const auto &req = request.toObject();
-    const auto source = Source(req[DAP_SOURCE].toObject());
+    const auto source = Source(req[DAP_SOURCE].toObject(), *m_msgContext);
     const int line = req[DAP_LINE].toInt();
     if (response.success) {
         Q_EMIT gotoTargets(source, line, GotoTarget::parseList(response.body.toObject()[QStringLiteral("targets")].toArray()));
@@ -425,7 +477,7 @@ void Client::write(const QJsonObject &msg)
 {
     const auto payload = QJsonDocument(msg).toJson();
 
-    qCDebug(DAPCLIENT) << "--> " << msg;
+    qCDebug(DAPCLIENT, " --> %s", payload.data());
 
     // write header
     m_bus->write(DAP_TPL_HEADER_FIELD.arg(QLatin1String(DAP_CONTENT_LENGTH)).arg(payload.size()).toLatin1());
@@ -488,7 +540,7 @@ void Client::requestInitialize()
 void Client::requestConfigurationDone()
 {
     if (m_state != State::Initialized) {
-        qCWarning(DAPCLIENT) << "trying to configure in an unexpected status";
+        qCWarning(DAPCLIENT, "trying to configure in an unexpected status");
         return;
     }
 
@@ -617,15 +669,18 @@ void Client::requestDisconnect(bool restart)
 
 void Client::requestSource(const Source &source)
 {
-    QJsonObject arguments{{DAP_SOURCE_REFERENCE, source.sourceReference.value_or(0)}};
-    const QJsonObject sourceArg{{DAP_SOURCE_REFERENCE, source.sourceReference.value_or(0)}, {DAP_PATH, source.path}};
+    auto reference = source.sourceReference.value_or(0);
+    if (reference == 0 && source.path.scheme() == Source::referenceScheme())
+        reference = source.path.path().toInt();
+    QJsonObject arguments{{DAP_SOURCE_REFERENCE, reference}};
+    const QJsonObject sourceArg{{DAP_SOURCE_REFERENCE, reference}, {DAP_PATH, m_msgContext->toRemote(source.path)}};
 
     arguments[DAP_SOURCE] = sourceArg;
 
     this->write(makeRequest(DAP_SOURCE, arguments, &Client::processResponseSource));
 }
 
-void Client::requestSetBreakpoints(const QString &path, const QList<SourceBreakpoint> &breakpoints, bool sourceModified)
+void Client::requestSetBreakpoints(const QUrl &path, const QList<SourceBreakpoint> &breakpoints, bool sourceModified)
 {
     requestSetBreakpoints(Source(path), breakpoints, sourceModified);
 }
@@ -636,7 +691,7 @@ void Client::requestSetBreakpoints(const Source &source, const QList<SourceBreak
     for (const auto &item : breakpoints) {
         bpoints.append(item.toJson());
     }
-    QJsonObject arguments{{DAP_SOURCE, source.toJson()}, {DAP_BREAKPOINTS, bpoints}, {QStringLiteral("sourceModified"), sourceModified}};
+    QJsonObject arguments{{DAP_SOURCE, source.toJson(*m_msgContext)}, {DAP_BREAKPOINTS, bpoints}, {QStringLiteral("sourceModified"), sourceModified}};
 
     this->write(makeRequest(QStringLiteral("setBreakpoints"), arguments, &Client::processResponseSetBreakpoints));
 }
@@ -659,14 +714,14 @@ void Client::requestWatch(const QString &expression, std::optional<int> frameId)
     requestEvaluate(expression, QStringLiteral("watch"), frameId);
 }
 
-void Client::requestGotoTargets(const QString &path, const int line, const std::optional<int> column)
+void Client::requestGotoTargets(const QUrl &path, const int line, const std::optional<int> column)
 {
     requestGotoTargets(Source(path), line, column);
 }
 
 void Client::requestGotoTargets(const Source &source, const int line, const std::optional<int> column)
 {
-    QJsonObject arguments{{DAP_SOURCE, source.toJson()}, {DAP_LINE, line}};
+    QJsonObject arguments{{DAP_SOURCE, source.toJson(*m_msgContext)}, {DAP_LINE, line}};
     if (column) {
         arguments[DAP_COLUMN] = *column;
     }
@@ -677,7 +732,7 @@ void Client::requestGotoTargets(const Source &source, const int line, const std:
 void Client::requestLaunchCommand()
 {
     if (m_state != State::Initializing) {
-        qCWarning(DAPCLIENT) << "trying to launch in an unexpected state";
+        qCWarning(DAPCLIENT, "trying to launch in an unexpected state");
         return;
     }
     if (m_launchCommand.isNull() || m_launchCommand.isEmpty())
@@ -703,30 +758,90 @@ void Client::checkRunning()
     }
 }
 
-void Client::onServerOutput(const QString &message)
+void Client::onServerOutput(const QByteArray &message)
 {
-    Q_EMIT outputProduced(dap::Output(message, dap::Output::Category::Console));
+    Q_EMIT outputProduced(dap::Output(QString::fromLocal8Bit(message), dap::Output::Category::Console));
 }
 
-void Client::onProcessOutput(const QString &message)
+void Client::onProcessOutput(const QByteArray &_message)
 {
-    Q_EMIT outputProduced(dap::Output(message, dap::Output::Category::Stdout));
+    auto message = _message;
+    if (m_checkExtraData) {
+        if (m_outputMsgParser->m_buffer.isEmpty() && !message.startsWith(DAP_CONTENT_LENGTH)) {
+            m_checkExtraData = false;
+        } else {
+            m_outputMsgParser->push(message);
+            auto extra = m_outputMsgParser->read();
+            if (!extra.size()) {
+                // need more
+                return;
+            }
+            // only 1 attempt, so no more
+            m_checkExtraData = false;
+            processExtraData(extra);
+            // consider remainder
+            message = std::move(m_outputMsgParser->m_buffer);
+            if (!message.size())
+                return;
+        }
+    }
+    if (m_protocol.redirectStdout)
+        Q_EMIT outputProduced(dap::Output(QString::fromLocal8Bit(message), dap::Output::Category::Stdout));
 }
 
 QString Client::extractCommand(const QJsonObject &launchRequest)
 {
     const auto &command = launchRequest[DAP_COMMAND].toString();
     if ((command != DAP_LAUNCH) && (command != DAP_ATTACH)) {
-        qCWarning(DAPCLIENT) << "unsupported request command: " << command;
+        qCWarning(DAPCLIENT, "unsupported request command: %ls", qUtf16Printable(command));
         return QString();
     }
     return command;
 }
 
+void Client::processExtraData(const QByteArray &data)
+{
+    auto ctx = static_cast<ClientMessageContext *>(m_msgContext.get());
+    auto mapping = ctx->pathMap;
+    Q_ASSERT(mapping);
+    qCInfo(DAPCLIENT) << "process extra data" << data;
+    bool ok = Utils::updateMapping(*mapping, data);
+    qCInfo(DAPCLIENT) << "map updated" << ok << "now;\n" << *mapping;
+}
+
 void Client::read()
 {
-    m_buffer.append(m_bus->read());
+    m_msgParser->push(m_bus->read());
+    while (true) {
+        auto data = m_msgParser->read();
+        if (!data.size())
+            break;
 
+        // check oob data
+        if (m_checkExtraData && data.front() != '{' && !isblank(data.front())) {
+            /* this does not make for valid json, so treat as extra */
+            processExtraData(data);
+            m_checkExtraData = false;
+            continue;
+        }
+
+        // parse payload
+        QJsonParseError jsonError;
+        const auto message = QJsonDocument::fromJson(data, &jsonError);
+        if ((jsonError.error != QJsonParseError::NoError) || message.isNull() || !message.isObject()) {
+            qCWarning(DAPCLIENT, "JSON bad format: %ls", qUtf16Printable(jsonError.errorString()));
+            continue;
+        }
+
+        qDebug(DAPCLIENT, "<-- %s", message.toJson().constData());
+
+        // process message
+        processProtocolMessage(message.object());
+    }
+}
+
+QByteArray Client::MessageParser::read()
+{
     while (true) {
         // read headers
         const auto info = readHeader();
@@ -741,22 +856,13 @@ void Client::read()
         }
         m_buffer.remove(0, info->payloadStart + info->payloadLength);
 
-        // parse payload
-        QJsonParseError jsonError;
-        const auto message = QJsonDocument::fromJson(data, &jsonError);
-        if ((jsonError.error != QJsonParseError::NoError) || message.isNull() || !message.isObject()) {
-            qCWarning(DAPCLIENT) << "JSON bad format: " << jsonError.errorString();
-            continue;
-        }
-
-        qDebug(DAPCLIENT) << "<-- " << message;
-
-        // process message
-        processProtocolMessage(message.object());
+        return data;
     }
+
+    return {};
 }
 
-std::optional<Client::HeaderInfo> Client::readHeader()
+std::optional<Client::HeaderInfo> Client::MessageParser::readHeader()
 {
     int length = -1;
     int start = 0;
@@ -785,7 +891,7 @@ std::optional<Client::HeaderInfo> Client::readHeader()
         if (header.size() == 0) {
             if (length < 0) {
                 // unexpected end of header
-                qCWarning(DAPCLIENT) << "unexpected end of header block";
+                qCWarning(DAPCLIENT, "unexpected end of header block");
                 discardExploredBuffer();
                 continue;
             }
@@ -795,7 +901,7 @@ std::optional<Client::HeaderInfo> Client::readHeader()
         // parse field
         const int sep = header.indexOf(":");
         if (sep < 0) {
-            qCWarning(DAPCLIENT) << "cannot parse header field: " << header;
+            qCWarning(DAPCLIENT, "cannot parse header field: %s", header.constData());
             discardExploredBuffer();
             continue; // CONTINUE HEADER
         }
@@ -805,7 +911,7 @@ std::optional<Client::HeaderInfo> Client::readHeader()
             bool ok = false;
             length = header.mid(sep + 1, header.size() - sep).toInt(&ok);
             if (!ok) {
-                qCWarning(DAPCLIENT) << "invalid value: " << header;
+                qCWarning(DAPCLIENT, "invalid value: %s", header.constData());
                 discardExploredBuffer();
                 continue; // CONTINUE HEADER
             }
@@ -825,7 +931,7 @@ void Client::start()
     m_launched = false;
     m_configured = false;
     if (m_state != State::None) {
-        qCWarning(DAPCLIENT) << "trying to re-start has no effect";
+        qCWarning(DAPCLIENT, "trying to re-start has no effect");
         return;
     }
     requestInitialize();
