@@ -16,6 +16,8 @@
 #include "entities.h"
 #include "settings.h"
 
+#include <exec_utils.h>
+
 namespace dap
 {
 class Client : public QObject
@@ -31,9 +33,9 @@ public:
     };
     Q_ENUM(State)
 
-    Client(const settings::ProtocolSettings &protocolSettings, Bus *bus, QObject *parent = nullptr);
+    Client(const settings::ProtocolSettings &protocolSettings, Bus *bus, Utils::PathMappingPtr, QObject *parent = nullptr);
 
-    Client(const settings::ClientSettings &clientSettings, QObject *parent = nullptr);
+    Client(const settings::ClientSettings &clientSettings, Utils::PathMappingPtr, QObject *parent = nullptr);
 
     ~Client() override;
 
@@ -73,11 +75,11 @@ public:
     void requestTerminate(bool restart = false);
     void requestDisconnect(bool restart = false);
     void requestSource(const Source &source);
-    void requestSetBreakpoints(const QString &path, const QList<dap::SourceBreakpoint> &breakpoints, bool sourceModified = false);
+    void requestSetBreakpoints(const QUrl &path, const QList<dap::SourceBreakpoint> &breakpoints, bool sourceModified = false);
     void requestSetBreakpoints(const dap::Source &source, const QList<dap::SourceBreakpoint> &breakpoints, bool sourceModified = false);
     void requestEvaluate(const QString &expression, const QString &context, std::optional<int> frameId = std::nullopt);
     void requestWatch(const QString &expression, std::optional<int> frameId = std::nullopt);
-    void requestGotoTargets(const QString &path, int line, std::optional<int> column = std::nullopt);
+    void requestGotoTargets(const QUrl &path, int line, std::optional<int> column = std::nullopt);
     void requestGotoTargets(const dap::Source &source, int line, std::optional<int> column = std::nullopt);
     void requestHotReload();
     void requestHotRestart();
@@ -118,8 +120,8 @@ Q_SIGNALS:
     void variables(int variablesReference, const QList<Variable> &);
     void modules(const ModulesInfo &);
     void serverDisconnected();
-    void sourceContent(const QString &path, int reference, const SourceContent &content);
-    void sourceBreakpoints(const QString &path, int reference, const std::optional<QList<Breakpoint>> &breakpoints);
+    void sourceContent(const QUrl &path, int reference, const SourceContent &content);
+    void sourceBreakpoints(const QUrl &path, int reference, const std::optional<QList<Breakpoint>> &breakpoints);
     void breakpointChanged(const BreakpointEvent &);
     void expressionEvaluated(const QString &expression, const std::optional<EvaluateInfo> &);
     void gotoTargets(const Source &source, int line, const QList<GotoTarget> &targets);
@@ -187,8 +189,10 @@ private:
     void requestLaunchCommand();
 
     void checkRunning();
-    void onServerOutput(const QString &message);
-    void onProcessOutput(const QString &message);
+    void onServerOutput(const QByteArray &message);
+    void onProcessOutput(const QByteArray &message);
+
+    void processExtraData(const QByteArray &data);
 
     /*
      * server capabilities
@@ -197,7 +201,6 @@ private:
 
     Bus *m_bus = nullptr;
     bool m_managedBus;
-    QByteArray m_buffer;
     int m_seq = 0;
 
     struct Request {
@@ -213,6 +216,13 @@ private:
 
     settings::ProtocolSettings m_protocol;
     QString m_launchCommand;
+    std::unique_ptr<MessageContext> m_msgContext;
+
+    class MessageParser;
+    // unique_ptr does not work well with undefined type on some platforms
+    MessageParser *const m_msgParser = nullptr;
+    bool m_checkExtraData = false;
+    MessageParser *const m_outputMsgParser = nullptr;
 };
 
 }
