@@ -19,6 +19,18 @@ class LocalsView : public QTreeWidget
 {
     Q_OBJECT
 public:
+    enum Role {
+        VariableReference = Qt::UserRole + 1,
+    };
+    enum Type {
+        PendingDataItem = QTreeWidgetItem::UserType + 1
+    };
+    enum Column {
+        Column_Symbol = 0,
+        Column_Type = 1,
+        Column_Value = 2,
+    };
+
     LocalsView(QWidget *parent = nullptr);
     ~LocalsView() override;
 
@@ -30,6 +42,7 @@ public Q_SLOTS:
 
 Q_SIGNALS:
     void localsVisible(bool visible);
+    void requestVariable(int variableReference);
 
 protected:
     void showEvent(QShowEvent *event) override;
@@ -38,6 +51,8 @@ protected:
 private:
     QTreeWidgetItem *createWrappedItem(QTreeWidgetItem *parent, const dap::Variable &variable);
     QTreeWidgetItem *createWrappedItem(QTreeWidget *parent, const dap::Variable &variable);
+    void onContextMenu(QPoint pos);
+    void onItemExpanded(QTreeWidgetItem *);
 
     QHash<int, QTreeWidgetItem *> m_variables;
 };
