@@ -36,10 +36,11 @@
 
 #include "debugconfigpage.h"
 #include <KShell>
-#include <KTextEditor/Document>
 #include <ktexteditor/editor.h>
 #include <ktexteditor/view.h>
 
+#include <ktexteditor_utils.h>
+
 K_PLUGIN_FACTORY_WITH_JSON(KatePluginGDBFactory, "kategdbplugin.json", registerPlugin<KatePluginGDB>();)
 
 static const QString CONFIG_DEBUGPLUGIN{QStringLiteral("debugplugin")};
@@ -185,7 +186,7 @@ KatePluginGDBView::KatePluginGDBView(KatePluginGDB *plugin, KTextEditor::MainWin
     auto *variableContainer = new QWidget();
     auto *variableLayout = new QVBoxLayout(variableContainer);
     m_scopeCombo = new QComboBox();
-    connect(m_scopeCombo, QOverload<int>::of(&QComboBox::currentIndexChanged), this, &KatePluginGDBView::scopeSelected);
+    connect(m_scopeCombo, &QComboBox::currentIndexChanged, this, &KatePluginGDBView::scopeSelected);
     m_localsView = new LocalsView();
     variableLayout->addWidget(m_scopeCombo);
     variableLayout->addWidget(m_localsView);
@@ -434,6 +435,20 @@ void KatePluginGDBView::writeSessionConfig(KConfigGroup &config)
 void KatePluginGDBView::slotDebug()
 {
     initDebugToolview();
+    // First, get the configuration from the ConfigView.
+    QString errorMessage;
+    const auto dbgConfOpt = m_configView->currentDAPTarget(true, errorMessage);
+
+    // Now, check if the configuration is valid before doing anything else.
+    // Our previous fix in configview.cpp clears the debugger name to signal an error.
+    if (!errorMessage.isEmpty()) {
+        // The user has already been shown a warning message inside currentDAPTarget().
+        // We just need to abort the entire launch process now, which this return does.
+        const QIcon icon = QIcon::fromTheme(QStringLiteral("debug-run"));
+        const QString message = i18n("Error: %1", errorMessage);
+        Utils::showMessage(message, icon, QStringLiteral("Debug"), MessageType::Error, m_mainWin);
+        return;
+    }
 
     m_outputArea->clear();
 
@@ -445,7 +460,7 @@ void KatePluginGDBView::slotDebug()
     m_scopeCombo->clear();
     m_localsView->clear();
 
-    m_backend->runDebugger(m_configView->currentDAPTarget(true));
+    m_backend->runDebugger(dbgConfOpt);
 }
 
 void KatePluginGDBView::slotRestart()
@@ -560,7 +575,7 @@ void KatePluginGDBView::slotGoTo(const QUrl &url, int lineNum)
     }
 
     // skip not existing files
-    if (!QFile::exists(url.toLocalFile())) {
+    if (url.isLocalFile() && !QFile::exists(url.toLocalFile())) {
         m_lastExecLine = -1;
         return;
     }
@@ -568,10 +583,7 @@ void KatePluginGDBView::slotGoTo(const QUrl &url, int lineNum)
     m_lastExecUrl = url;
     m_lastExecLine = lineNum;
 
-    KTextEditor::View *editView = m_mainWin->openUrl(m_lastExecUrl);
-    editView->setCursorPosition(KTextEditor::Cursor(m_lastExecLine, 0));
-    m_mainWin->window()->raise();
-    m_mainWin->window()->setFocus();
+    Utils::goToDocumentLocation(m_mainWin, url, KTextEditor::Range({lineNum, 0}, 0), {.focus = true, .record = false});
 }
 
 void KatePluginGDBView::enableDebugActions(bool enable)
@@ -841,12 +853,12 @@ QString KatePluginGDBView::currentWord()
 {
     KTextEditor::View *kv = m_mainWin->activeView();
     if (!kv) {
-        qDebug() << "no KTextEditor::View";
+        qDebug("no KTextEditor::View");
         return QString();
     }
 
     if (!kv->cursorPosition().isValid()) {
-        qDebug() << "cursor not valid!";
+        qDebug("cursor not valid!");
         return QString();
     }
 
@@ -880,7 +892,7 @@ QString KatePluginGDBView::currentWord()
         endPos++;
     }
     if (startPos == endPos) {
-        qDebug() << "no word found!";
+        qDebug("no word found!");
         return QString();
     }
 
@@ -1115,21 +1127,18 @@ void KatePluginGDBView::onStackTreeContextMenuRequest(QPoint pos)
     if (index.isValid()) {
         auto frame = index.data(StackFrameModel::StackFrameRole).value<dap::StackFrame>();
         if (frame.source) {
-            auto url = QUrl::fromLocalFile(frame.source->path);
+            auto url = frame.source->path;
             int line = frame.line - 1;
             if (url.isValid()) {
                 auto a = menu.addAction(i18n("Open Location"));
                 connect(a, &QAction::triggered, m_stackTree, [this, url, line] {
-                    auto view = m_mainWin->openUrl(url);
-                    if (line >= 0) {
-                        view->setCursorPosition({line, 0});
-                    }
+                    Utils::goToDocumentLocation(m_mainWin, url, KTextEditor::Range({line, 0}, 0), {.focus = true, .record = false});
                 });
             }
         }
 
         auto a = menu.addAction(i18n("Copy Location"));
-        QString location = QStringLiteral("%1:%2").arg(frame.source->path).arg(frame.line);
+        QString location = QStringLiteral("%1:%2").arg(Utils::formatUrl(frame.source->path)).arg(frame.line);
         connect(a, &QAction::triggered, m_stackTree, [location] {
             qApp->clipboard()->setText(location);
         });
