@@ -6,6 +6,7 @@
 
 #pragma once
 
+#include "gitblameparser.h"
 #include "gitblametooltip.h"
 
 #include <KTextEditor/InlineNoteProvider>
@@ -23,18 +24,11 @@
 #include <QUrl>
 #include <QVariant>
 
-enum class KateGitBlameMode { None, SingleLine, AllLines, Count = AllLines };
-
-struct CommitInfo {
-    QByteArray hash;
-    QString authorName;
-    QDateTime authorDate;
-    QByteArray summary;
-};
-
-struct BlamedLine {
-    QByteArray shortCommitHash;
-    QByteArray lineText;
+enum class KateGitBlameMode {
+    None,
+    SingleLine,
+    AllLines,
+    Count = AllLines
 };
 
 class KateGitBlamePluginView;
@@ -63,7 +57,7 @@ private:
 class KateGitBlamePlugin : public KTextEditor::Plugin
 {
 public:
-    explicit KateGitBlamePlugin(QObject *parent = nullptr, const QVariantList & = QVariantList());
+    explicit KateGitBlamePlugin(QObject *parent);
 
     QObject *createView(KTextEditor::MainWindow *mainWindow) override;
 };
@@ -74,31 +68,39 @@ public:
     KateGitBlamePluginView(KateGitBlamePlugin *plugin, KTextEditor::MainWindow *mainwindow);
     ~KateGitBlamePluginView() override;
 
-    QPointer<KTextEditor::View> activeView() const;
-    QPointer<KTextEditor::Document> activeDocument() const;
+    KTextEditor::View *activeView() const;
+    KTextEditor::Document *activeDocument() const;
 
     bool hasBlameInfo() const;
-
     const CommitInfo &blameInfo(int lineNr);
 
     void showCommitInfo(const QString &hash, KTextEditor::View *view);
 
-    void setToolTipIgnoreKeySequence(QKeySequence sequence);
+    void setToolTipIgnoreKeySequence(const QKeySequence &sequence);
 
     void showCommitTreeView(const QUrl &url);
 
 private:
+    enum Command {
+        RevParse,
+        Config,
+        Blame,
+        IgnoreRevsFile
+    };
+    Q_ENUM(Command)
+
     void sendMessage(const QString &text, bool error);
 
+    void documentReloaded();
     void startGitBlameForActiveView();
 
     void startBlameProcess(const QUrl &url);
-    void blameFinished(int exitCode, QProcess::ExitStatus exitStatus);
+    void commandFinished(int exitCode, QProcess::ExitStatus exitStatus);
+    void parseGitBlameStdOutput();
 
     void startShowProcess(const QUrl &url, const QString &hash);
     void showFinished(int exitCode, QProcess::ExitStatus exitStatus);
-
-    const CommitInfo &blameGetUpdateInfo(int lineNr);
+    void onErrorOccurred(QProcess::ProcessError e);
 
     KTextEditor::MainWindow *m_mainWindow;
 
@@ -106,9 +108,7 @@ private:
 
     QProcess m_blameInfoProc;
     QProcess m_showProc;
-    QHash<QByteArray, CommitInfo> m_blameInfoForHash;
-    std::vector<BlamedLine> m_blamedLines;
-    QUrl m_blameUrl;
+
     QPointer<KTextEditor::View> m_lastView;
     int m_lineOffset{0};
     GitBlameTooltip m_tooltip;
@@ -116,4 +116,12 @@ private:
     class CommitDiffTreeView *m_commitFilesView;
     QPointer<KTextEditor::View> m_diffView;
     QTimer m_startBlameTimer;
+    QString m_parentPath;
+    Command m_currentCommand;
+    QString m_root;
+    QString m_ignoreRevsFile;
+    QString m_absoluteFilePath;
+
+    KateGitBlameParser m_parser;
+    QMetaObject::Connection m_documentReloadConnection;
 };
