@@ -53,9 +53,15 @@ public:
 public Q_SLOTS:
     void setFilterText(const QString &text)
     {
+#if QT_VERSION >= QT_VERSION_CHECK(6, 10, 0)
+        beginFilterChange();
+#endif
         m_filterStrings = text.split(QLatin1Char(' '), Qt::SkipEmptyParts);
-
+#if QT_VERSION >= QT_VERSION_CHECK(6, 10, 0)
+        endFilterChange(QSortFilterProxyModel::Direction::Rows);
+#else
         invalidateFilter();
+#endif
     }
 
 private:
@@ -173,7 +179,7 @@ GotoSymbolWidget::GotoSymbolWidget(KTextEditor::MainWindow *mainWindow, KateCTag
     connect(m_proxyModel, &QSortFilterProxyModel::rowsInserted, this, &GotoSymbolWidget::reselectFirst);
     connect(m_proxyModel, &QSortFilterProxyModel::rowsRemoved, this, &GotoSymbolWidget::reselectFirst);
 
-    QVBoxLayout *layout = new QVBoxLayout();
+    auto *layout = new QVBoxLayout();
     layout->setSpacing(0);
     layout->setContentsMargins(4, 4, 4, 4);
     layout->addWidget(m_lineEdit);
@@ -187,7 +193,7 @@ GotoSymbolWidget::GotoSymbolWidget(KTextEditor::MainWindow *mainWindow, KateCTag
 bool GotoSymbolWidget::eventFilter(QObject *obj, QEvent *event)
 {
     if (event->type() == QEvent::KeyPress || event->type() == QEvent::ShortcutOverride) {
-        QKeyEvent *keyEvent = static_cast<QKeyEvent *>(event);
+        auto *keyEvent = static_cast<QKeyEvent *>(event);
         if (obj == m_lineEdit) {
             const bool forward2list = (keyEvent->key() == Qt::Key_Up) || (keyEvent->key() == Qt::Key_Down) || (keyEvent->key() == Qt::Key_PageUp)
                 || (keyEvent->key() == Qt::Key_PageDown);
@@ -252,7 +258,7 @@ void GotoSymbolWidget::loadGlobalSymbols(const QString &text)
         return;
     }
 
-    QString currentWord = text;
+    const QString &currentWord = text;
     Tags::TagList list = Tags::getPartialMatchesNoi8n(m_tagFile, currentWord);
 
     if (list.isEmpty()) {
@@ -369,17 +375,17 @@ void GotoSymbolWidget::updateViewGeometry()
 
     const int rowCount = mode == Global ? m_globalSymbolsModel->rowCount() : m_symbolsModel->rowCount();
 
-    const QSize viewSize(width, std::min(std::max(rowHeight * rowCount + 2 * frameWidth, rowHeight * 6), viewMaxSize.height()));
+    const QSize viewSize(width, std::min(std::max((rowHeight * rowCount) + (2 * frameWidth), rowHeight * 6), viewMaxSize.height()));
 
     // Position should be central over the editor area, so map to global from
     // parent of central widget since the view is positioned in global coords
     const QPoint centralWidgetPos = window->parent() ? window->mapToGlobal(window->pos()) : window->pos();
-    const int xPos = std::max(0, centralWidgetPos.x() + (centralSize.width() - viewSize.width()) / 2);
-    const int yPos = std::max(0, centralWidgetPos.y() + (centralSize.height() - viewSize.height()) * 1 / 4);
+    const int xPos = std::max(0, centralWidgetPos.x() + ((centralSize.width() - viewSize.width()) / 2));
+    const int yPos = std::max(0, centralWidgetPos.y() + ((centralSize.height() - viewSize.height()) * 1 / 4));
 
     move(xPos, yPos);
 
-    QPropertyAnimation *animation = new QPropertyAnimation(this, "size");
+    auto *animation = new QPropertyAnimation(this, "size");
     animation->setDuration(150);
     animation->setStartValue(this->size());
     animation->setEndValue(viewSize);
