@@ -97,16 +97,23 @@ QSize InlayHintNoteProvider::inlineNoteSize(const KTextEditor::InlineNote &note)
     const auto &hints = this->hints();
     auto it = binaryFind(hints, note.position());
     if (it == hints.end()) {
-        qWarning() << Q_FUNC_INFO << note.view()->document()->documentName() << "failed to find note in m_hints, Note.position:" << note.position();
+        qWarning("%s %ls failed to find note in m_hints, Note.position: %ls",
+                 Q_FUNC_INFO,
+                 qUtf16Printable(note.view()->document()->documentName()),
+                 qUtf16Printable(note.position().toString()));
         return {};
     }
 
     const LSPInlayHint &hint = *it;
     const int padding = (hint.paddingLeft || hint.paddingRight) ? 4 : 0;
-    if (hint.width == 0) {
-        const auto font = qApp->font();
+
+    const auto font = note.font();
+    const auto fontSize = font.pointSize();
+    // invalidate cache when font size changes
+    if (hint.fontSize != fontSize) {
         const auto fm = QFontMetrics(font);
-        const_cast<LSPInlayHint &>(hint).width = fm.horizontalAdvance(hint.label) + padding;
+        hint.fontSize = fontSize;
+        hint.width = fm.horizontalAdvance(hint.label) + padding;
     }
     return {hint.width, note.lineHeight()};
 }
@@ -116,7 +123,7 @@ void InlayHintNoteProvider::paintInlineNote(const KTextEditor::InlineNote &note,
     const auto &hints = this->hints();
     auto it = binaryFind(hints, note.position());
     if (it != hints.end()) {
-        const auto font = qApp->font();
+        const auto font = note.font();
         painter.setFont(font);
         QRectF r{0., 0., (qreal)it->width, (qreal)note.lineHeight()};
 
@@ -286,12 +293,9 @@ void InlayHintsManager::sendRequest(KTextEditor::Range rangeToRequest)
             // and send over inlay hints for the full document anyways. To avoid issues, we
             // remove all the inlay hints that fall outside the range we requested hints for.
             if (rangeToRequest.isValid()) {
-                hints.erase(std::remove_if(hints.begin(),
-                                           hints.end(),
-                                           [rangeToRequest](const LSPInlayHint &h) {
-                                               return !rangeToRequest.contains(h.position);
-                                           }),
-                            hints.end());
+                std::erase_if(hints, [rangeToRequest](const LSPInlayHint &h) {
+                    return !rangeToRequest.contains(h.position);
+                });
             }
 
             const auto result = insertHintsForDoc(v->document(), rangeToRequest, hints);
@@ -469,16 +473,13 @@ void InlayHintsManager::onUnwrapped(KTextEditor::Document *doc, int line)
 
 void InlayHintsManager::clearHintsForDoc(KTextEditor::Document *doc)
 {
-    m_hintDataByDoc.erase(std::remove_if(m_hintDataByDoc.begin(),
-                                         m_hintDataByDoc.end(),
-                                         [doc](const HintData &hd) {
-                                             if (!doc) {
-                                                 // remove all null docs and docs where checksum doesn't match
-                                                 return !hd.doc || (hd.doc->checksum() != hd.checksum);
-                                             }
-                                             return hd.doc == doc;
-                                         }),
-                          m_hintDataByDoc.end());
+    std::erase_if(m_hintDataByDoc, [doc](const HintData &hd) {
+        if (!doc) {
+            // remove all null docs and docs where checksum doesn't match
+            return !hd.doc || (hd.doc->checksum() != hd.checksum);
+        }
+        return hd.doc == doc;
+    });
 }
 
 InlayHintsManager::InsertResult
@@ -538,7 +539,7 @@ InlayHintsManager::insertHintsForDoc(KTextEditor::Document *doc, KTextEditor::Ra
         return false;
     };
     // remove existing hints that contain affectedLines
-    existing.erase(std::remove_if(existing.begin(), existing.end(), pred), existing.end());
+    std::erase_if(existing, pred);
 
     // now add new ones
     affectedLines.clear();
