@@ -988,7 +988,7 @@ public:
         bool enabled = false;
         switch (kind) {
         case RangeData::KindEnum::Text: {
-            // well, it's a bit like searching for something, so re-use that color
+            // well, it's a bit like searching for something, so reuse that color
             static KTextEditor::Attribute::Ptr searchAttr;
             if (!searchAttr) {
                 searchAttr = new KTextEditor::Attribute();
@@ -1029,7 +1029,7 @@ public:
         }
 
         if (!attr) {
-            qWarning() << "Unexpected null attr";
+            qWarning("Unexpected null attr");
         }
 
         // highlight the range
@@ -1091,55 +1091,10 @@ public:
         addMarksRec(doc, treeModel->invisibleRootItem(), oranges, odocs);
     }
 
-    void goToDocumentLocation(const QUrl &uri, const KTextEditor::Range &location)
+    void goToDocumentLocation(const QUrl &uri, const std::optional<KTextEditor::Range> &location)
     {
-        int line = location.start().line();
-        int column = location.start().column();
-        KTextEditor::View *activeView = m_mainWindow->activeView();
-        if (!activeView || uri.isEmpty() || line < 0 || column < 0) {
-            return;
-        }
-
-        KTextEditor::Document *document = activeView->document();
-        KTextEditor::Cursor cdef(line, column);
-
-        KTextEditor::View *targetView = nullptr;
-        if (document && uri == document->url()) {
-            targetView = activeView;
-        } else {
-            targetView = m_mainWindow->openUrl(uri);
-        }
-        if (targetView) {
-            // save current position for location history
-            Utils::addPositionToHistory(activeView->document()->url(), activeView->cursorPosition(), m_mainWindow);
-            // save the position to which we are jumping in location history
-            Utils::addPositionToHistory(targetView->document()->url(), cdef, m_mainWindow);
-            targetView->setCursorPosition(cdef);
-            highlightLandingLocation(targetView, location);
-        }
-    }
-
-    /**
-     * @brief give a short 1sec temporary highlight where you land
-     */
-    void highlightLandingLocation(KTextEditor::View *view, const KTextEditor::Range &location)
-    {
-        if (!m_highlightGoto || !m_highlightGoto->isChecked()) {
-            return;
-        }
-        auto doc = view->document();
-        if (!doc) {
-            return;
-        }
-        auto mr = doc->newMovingRange(location);
-        KTextEditor::Attribute::Ptr attr(new KTextEditor::Attribute);
-        attr->setUnderlineStyle(QTextCharFormat::SingleUnderline);
-        mr->setView(view);
-        mr->setAttribute(attr);
-        QTimer::singleShot(1000, doc, [mr] {
-            mr->setRange(KTextEditor::Range::invalid());
-            delete mr;
-        });
+        bool hl = location && !location->isEmpty() && m_highlightGoto && m_highlightGoto->isChecked();
+        Utils::goToDocumentLocation(m_mainWindow, uri, location, {.highlight = hl});
     }
 
     static QModelIndex getPrimaryModelIndex(QModelIndex index)
@@ -1736,7 +1691,7 @@ public:
             qCInfo(LSPCLIENT) << "applying edit" << edit.label;
             applyWorkspaceEdit(edit.edit, nullptr);
         } else {
-            qCInfo(LSPCLIENT) << "ignoring edit";
+            qCInfo(LSPCLIENT, "ignoring edit");
         }
         h({.applied = m_accept_edit, .failureReason = QString()});
     }
@@ -1932,9 +1887,9 @@ public:
             return;
         }
 
-        auto h = [this](const QString &reply) {
+        auto h = [this](const QUrl &reply) {
             if (!reply.isEmpty()) {
-                m_mainWindow->openUrl(QUrl(reply));
+                goToDocumentLocation(reply, std::nullopt);
             } else {
                 showMessage(i18n("Corresponding Header/Source not found"), KTextEditor::Message::Information);
             }
@@ -2122,7 +2077,7 @@ public:
         }
 
         // host application will handle these message for us, including auto-show settings
-        Utils::showMessage(genericMessage, m_mainWindow);
+        m_mainWindow->showMessage(genericMessage);
     }
 
     // params type is same for show or log and is treated the same way
@@ -2217,7 +2172,7 @@ public:
             return;
         }
 
-        // NOTE the intendation mode should probably be set to None,
+        // NOTE the indentation mode should probably be set to None,
         // so as not to experience unpleasant interference
         auto cursor = activeView->cursorPosition();
         QChar lastChar = cursor.column() == 0 ? QChar::fromLatin1('\n') : doc->characterAt({cursor.line(), cursor.column() - 1});
@@ -2420,13 +2375,13 @@ public:
         bool registered = m_completionViews.contains(view);
 
         if (!registered && server && server->capabilities().completionProvider.provider) {
-            qCInfo(LSPCLIENT) << "registering cci";
+            qCInfo(LSPCLIENT, "registering cci");
             view->registerCompletionModel(m_completion.get());
             m_completionViews.append(view);
         }
 
         if (registered && !server) {
-            qCInfo(LSPCLIENT) << "unregistering cci";
+            qCInfo(LSPCLIENT, "unregistering cci");
             view->unregisterCompletionModel(m_completion.get());
             m_completionViews.removeAll(view);
         }
