@@ -450,10 +450,22 @@ struct LSPInlayHint {
     // QString tooltip;
 
     // kate specific
-    int width = 0; ///> Used to cache width
+    // used to cache width
+    mutable int fontSize = 0;
+    mutable int width = 0;
 };
 
 struct LSPMessageRequestAction {
     QString title;
     std::function<void()> choose;
 };
+
+// workspace/configuration request structures (LSP 3.6+)
+struct LSPConfigurationItem {
+    std::optional<QString> scopeUri;
+    std::optional<QString> section;
+};
+
+struct LSPConfigurationParams {
+    QList<LSPConfigurationItem> items;
+};
