@@ -77,6 +77,10 @@ Q_SIGNALS:
     void serverShowMessage(LSPClientServer *server, const LSPShowMessageParams &);
     void serverLogMessage(LSPClientServer *server, const LSPShowMessageParams &);
     void serverWorkDoneProgress(LSPClientServer *server, const LSPWorkDoneProgressParams &);
+    void showMessageRequest(const LSPShowMessageParams &message,
+                            const QList<LSPMessageRequestAction> &actions,
+                            const std::function<void()> chooseNothing,
+                            bool &handled);
 };
 
 class LSPClientRevisionSnapshot : public QObject
@@ -85,9 +89,5 @@ class LSPClientRevisionSnapshot : public QObject
 
 public:
     // find a locked revision for url in snapshot
-#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
     virtual void find(const QUrl &url, KTextEditor::Document *&doc, qint64 &revision) const = 0;
-#else
-    virtual void find(const QUrl &url, KTextEditor::MovingInterface *&miface, qint64 &revision) const = 0;
-#endif
 };
