@@ -45,6 +45,7 @@ private Q_SLOTS:
     void testTabsKeepOrderOnRestore2();
     void testTabsKeepOrderOnRestore3();
     void testShowMessageWorks();
+    void testPinnedDocs();
 
 private:
     std::unique_ptr<QTemporaryDir> m_tempdir;
@@ -213,7 +214,7 @@ void KateViewManagementTest2::testMultipleViewCursorPositionIsRestored()
     {
         QCommandLineParser parser;
         // TODO: reuse option from kate/main.cpp properly
-        const QCommandLineOption startSessionOption(QStringList() << QStringLiteral("s") << QStringLiteral("start"),
+        const QCommandLineOption startSessionOption(QStringList{QStringLiteral("s"), QStringLiteral("start")},
                                                     i18n("Start Kate with a given session."),
                                                     i18n("session"));
         parser.addOption(startSessionOption);
@@ -403,7 +404,7 @@ void KateViewManagementTest2::testShowMessageWorks()
 {
     app->sessionManager()->sessionNew();
 
-    auto outputView = app->activeMainWindow()->parent()->findChild<QWidget *>("KateOutputView");
+    auto outputView = app->activeMainWindow()->parent()->findChild<QWidget *>(QStringLiteral("KateOutputView"));
     QVERIFY(outputView);
     auto outputViewTextBrowser = outputView->findChild<QTextBrowser *>();
     QVERIFY(outputViewTextBrowser);
@@ -416,16 +417,66 @@ void KateViewManagementTest2::testShowMessageWorks()
 
     app->activeMainWindow()->showMessage(msg);
     Utils::showMessage(QStringLiteral("Test KTextEditor::utils::showMessage"), QIcon(), QStringLiteral("Test"), MessageType::Error);
-    msg.insert(QStringLiteral("text"), QStringLiteral("Test2 testShowMessageWorks"));
-    Utils::showMessage(msg);
 
     QVERIFY(outputViewTextBrowser->toPlainText().contains(QStringLiteral("Test testShowMessageWorks")));
     QVERIFY(outputViewTextBrowser->toPlainText().contains(QStringLiteral("Test KTextEditor::utils::showMessage")));
-    QVERIFY(outputViewTextBrowser->toPlainText().contains(QStringLiteral("Test2 testShowMessageWorks")));
 
     QVERIFY(outputView->isVisible());
 }
 
+void KateViewManagementTest2::testPinnedDocs()
+{
+    QTemporaryFile f1;
+    QVERIFY(f1.open());
+    f1.write(QStringLiteral("Some rando text").toUtf8());
+    f1.close();
+    const QUrl url = QUrl::fromLocalFile(f1.fileName());
+
+    // Pin document
+    {
+        app->sessionManager()->sessionNew();
+        app->activeKateMainWindow()->openUrl(url);
+
+        QMenu menu;
+        app->activeKateMainWindow()->viewManager()->activeViewSpace()->buildContextMenu(0, menu);
+        auto a = getAction(menu, "Pin Document");
+        QVERIFY(a);
+        QCOMPARE(a->icon().name(), QStringLiteral("pin"));
+
+        auto pinAction = app->activeKateMainWindow()->action(QStringLiteral("pin_active_document"));
+        QVERIFY(pinAction);
+        pinAction->trigger();
+
+        auto tabBar = app->activeKateMainWindow()->viewManager()->activeViewSpace()->m_tabBar;
+        QCOMPARE(tabBar->tabIcon(0).name(), QStringLiteral("pin"));
+    }
+
+    // Open new session, expect pinned document to be there
+    {
+        app->sessionManager()->activateSession(QStringLiteral("pinned doc session 1"), false, true);
+        // expect to find the pinned doc
+        QTRY_COMPARE(app->activeKateMainWindow()->activeView()->document()->url(), url);
+
+        QMenu menu;
+        app->activeKateMainWindow()->viewManager()->activeViewSpace()->buildContextMenu(0, menu);
+        auto unpinAction = getAction(menu, "Unpin Document");
+        QVERIFY(unpinAction);
+        QCOMPARE(unpinAction->icon().name(), QStringLiteral("window-unpin"));
+
+        // unpin the doc, expect the icon to change
+        unpinAction->trigger();
+
+        auto tabBar = app->activeKateMainWindow()->viewManager()->activeViewSpace()->m_tabBar;
+        QCOMPARE_NE(tabBar->tabIcon(0).name(), QStringLiteral("pin"));
+    }
+
+    // Open another session, expect no pinned document
+    {
+        app->sessionManager()->sessionNew();
+        QCOMPARE(app->activeKateMainWindow()->views().size(), 0);
+    }
+}
+
 QTEST_MAIN(KateViewManagementTest2)
 
 #include "kate_view_mgmt_test2.moc"
