@@ -10,6 +10,8 @@
 #include <QUrl>
 #include <QWidgetList>
 
+#include <optional>
+
 QT_BEGIN_NAMESPACE
 class QScrollBar;
 class QAction;
@@ -40,7 +42,6 @@ enum MessageType {
 
 namespace Utils
 {
-
 // A helper class that maintains scroll position
 struct KATE_PRIVATE_EXPORT KateScrollBarRestorer {
     explicit KateScrollBarRestorer(KTextEditor::View *view);
@@ -60,35 +61,54 @@ KATE_PRIVATE_EXPORT QFont editorFont();
 /**
  * Returns the range that is currently visible in the @p view
  */
-KATE_PRIVATE_EXPORT KTextEditor::Range getVisibleRange(KTextEditor::View *view);
+[[nodiscard]] KATE_PRIVATE_EXPORT KTextEditor::Range getVisibleRange(KTextEditor::View *view);
 
 /**
  * @brief Given path "/home/user/xyz.txt" returns "xyz.txt"
  */
-KATE_PRIVATE_EXPORT inline QString fileNameFromPath(const QString &path)
+[[nodiscard]] KATE_PRIVATE_EXPORT inline QString fileNameFromPath(const QString &path)
 {
     int lastSlash = path.lastIndexOf(QLatin1Char('/'));
     return lastSlash == -1 ? path : path.mid(lastSlash + 1);
 }
 
+[[nodiscard]] KATE_PRIVATE_EXPORT inline QString formatUrl(const QUrl &url)
+{
+    return url.toString(QUrl::PreferLocalFile | QUrl::RemovePassword);
+}
+
 /**
  * Return a matching icon for the given document.
  * We use the mime type icon for unmodified stuff and the modified one for modified docs.
  * @param doc document to get icon for
  * @return icon, always valid
  */
-KATE_PRIVATE_EXPORT QIcon iconForDocument(KTextEditor::Document *doc);
+[[nodiscard]] KATE_PRIVATE_EXPORT QIcon iconForDocument(KTextEditor::Document *doc);
 
-KATE_PRIVATE_EXPORT QAction *toolviewShowAction(KTextEditor::MainWindow *, const QString &toolviewName);
+[[nodiscard]] KATE_PRIVATE_EXPORT QAction *toolviewShowAction(KTextEditor::MainWindow *, const QString &toolviewName);
 
-KATE_PRIVATE_EXPORT QWidget *toolviewForName(KTextEditor::MainWindow *, const QString &toolviewName);
+[[nodiscard]] KATE_PRIVATE_EXPORT QWidget *toolviewForName(KTextEditor::MainWindow *, const QString &toolviewName);
 
-KATE_PRIVATE_EXPORT QWidget *activeToolviewForSide(KTextEditor::MainWindow *mainWindow, int side);
+[[nodiscard]] KATE_PRIVATE_EXPORT QWidget *activeToolviewForSide(KTextEditor::MainWindow *mainWindow, int side);
+
+// helper struct to support various keyword options below
+struct GoToOptions {
+    bool highlight = false;
+    bool focus = false;
+    bool record = true;
+};
+
+KATE_PRIVATE_EXPORT void
+goToDocumentLocation(KTextEditor::MainWindow *mainWindow, const QUrl &uri, const std::optional<KTextEditor::Range> &location, const GoToOptions &options = {});
+
+/**
+ * @brief give a short 1sec temporary highlight where you land
+ */
+KATE_PRIVATE_EXPORT void highlightLandingLocation(KTextEditor::View *view, const KTextEditor::Range &location);
 
 /*** BEGIN KTextEditor::MainWindow extensions **/
 KATE_PRIVATE_EXPORT void
 showMessage(const QString &message, const QIcon &icon, const QString &category, MessageType type, KTextEditor::MainWindow *mainWindow = nullptr);
-KATE_PRIVATE_EXPORT void showMessage(const QVariantMap &map, KTextEditor::MainWindow *mainWindow = nullptr);
 
 KATE_PRIVATE_EXPORT void showDiff(const QByteArray &diff, const DiffParams &params, KTextEditor::MainWindow *mainWindow);
 
@@ -96,7 +116,7 @@ KATE_PRIVATE_EXPORT void showDiff(const QByteArray &diff, const DiffParams &para
 /// TODO: Maybe it would be more versatile to return *all* widgets, including KTextEditor::Views, here, or perhaps
 ///       controlled by a flag to filter which widgets to return. Think e.g. about the document switching plugin.
 ///       activeWidget() (not yet "public") uses those semantics, already.
-KATE_PRIVATE_EXPORT QWidgetList widgets(KTextEditor::MainWindow *mainWindow);
+[[nodiscard]] KATE_PRIVATE_EXPORT QWidgetList widgets(KTextEditor::MainWindow *mainWindow);
 
 KATE_PRIVATE_EXPORT void insertWidgetInStatusbar(QWidget *widget, KTextEditor::MainWindow *mainWindow);
 
@@ -106,25 +126,34 @@ KATE_PRIVATE_EXPORT void addPositionToHistory(const QUrl &url, KTextEditor::Curs
 /**
  * Returns project base dir for provided document
  */
-KATE_PRIVATE_EXPORT QString projectBaseDirForDocument(KTextEditor::Document *doc);
+[[nodiscard]] KATE_PRIVATE_EXPORT QString projectBaseDirForDocument(KTextEditor::Document *doc);
 
 /**
  * Returns project map for provided document
  */
-KATE_PRIVATE_EXPORT QVariantMap projectMapForDocument(KTextEditor::Document *doc);
+[[nodiscard]] KATE_PRIVATE_EXPORT QVariantMap projectMapForDocument(KTextEditor::Document *doc);
 
 /**
  * Convert an url to a xxx.yyy @ dir representation, used e.g. in the window title
  */
-KATE_PRIVATE_EXPORT QString niceFileNameWithPath(const QUrl &url);
+[[nodiscard]] KATE_PRIVATE_EXPORT QString niceFileNameWithPath(const QUrl &url);
 
 /**
  * Convert an url to a normalize one, used by the document manager and Co.
  */
-KATE_PRIVATE_EXPORT QUrl normalizeUrl(QUrl url);
+[[nodiscard]] KATE_PRIVATE_EXPORT QUrl normalizeUrl(QUrl url);
 
 /**
  * Convert an url to an absolute one, used by the document manager and Co.
  */
-KATE_PRIVATE_EXPORT QUrl absoluteUrl(QUrl url);
+[[nodiscard]] KATE_PRIVATE_EXPORT QUrl absoluteUrl(QUrl url);
+
+[[nodiscard]] KATE_PRIVATE_EXPORT bool isDocumentPinned(KTextEditor::Document *doc);
+KATE_PRIVATE_EXPORT void togglePinDocument(KTextEditor::Document *document);
+
+/**
+ * Try to release memory to the system.
+ * Will call e.g. malloc_trim or similar stuff if feasible.
+ */
+KATE_PRIVATE_EXPORT void releaseMemoryToOperatingSystem();
 }
