@@ -18,6 +18,7 @@
 #include <QIcon>
 #include <QMimeDatabase>
 
+#include <memory_resource>
 #include <unordered_set>
 
 KateQuickOpenModel::KateQuickOpenModel(QObject *parent)
@@ -49,7 +50,7 @@ QVariant KateQuickOpenModel::data(const QModelIndex &idx, int role) const
     switch (role) {
     case Qt::DisplayRole:
     case Role::FileName:
-        return entry.fileName;
+        return QString::fromRawData(entry.fileName.data(), entry.fileName.size());
     case Role::FilePath: {
         // no .remove since that might remove all occurrence in rare cases
         const auto &path = entry.filePath;
@@ -64,7 +65,7 @@ QVariant KateQuickOpenModel::data(const QModelIndex &idx, int role) const
         return {};
     }
     case Qt::DecorationRole:
-        return QIcon::fromTheme(QMimeDatabase().mimeTypeForFile(entry.fileName, QMimeDatabase::MatchExtension).iconName());
+        return QIcon::fromTheme(QMimeDatabase().mimeTypeForFile(entry.filePath, QMimeDatabase::MatchExtension).iconName());
     case Qt::UserRole:
         return !entry.document ? QUrl::fromLocalFile(entry.filePath) : entry.document->url();
     case Role::Score:
@@ -81,12 +82,12 @@ QVariant KateQuickOpenModel::data(const QModelIndex &idx, int role) const
 void KateQuickOpenModel::refresh(KateMainWindow *mainWindow)
 {
     QObject *projectView = mainWindow->pluginView(QStringLiteral("kateprojectplugin"));
-    const auto sortedViews = mainWindow->viewManager()->views();
+    const auto sortedViews = mainWindow->views();
     const QList<KTextEditor::Document *> openDocs = KateApp::self()->documentManager()->documentList();
     const QStringList projectDocs = projectView
         ? (m_listMode == CurrentProject ? projectView->property("projectFiles") : projectView->property("allProjectsFiles")).toStringList()
         : QStringList();
-    const auto projects = projectView ? projectView->property("allProjects").toMap() : QVariantMap();
+    const auto projects = projectView ? projectView->property("allProjects").value<QMap<QString, QString>>() : QMap<QString, QString>();
     const QString projectBase = [projectView]() -> QString {
         if (!projectView) {
             return QString();
@@ -111,13 +112,17 @@ void KateQuickOpenModel::refresh(KateMainWindow *mainWindow)
     m_projectBase = projectBase;
 
     std::vector<ModelEntry> allDocuments;
+    std::vector<QString> strings;
     allDocuments.reserve(sortedViews.size() + projectDocs.size());
 
-    std::unordered_set<QString> openedDocUrls;
-    std::unordered_set<KTextEditor::Document *> seenDocuments;
+    std::byte tempBuffer[256 * 1000];
+    std::pmr::monotonic_buffer_resource memory(tempBuffer, sizeof(tempBuffer));
+
+    std::pmr::unordered_set<QString> openedDocUrls(&memory);
+    std::pmr::unordered_set<KTextEditor::Document *> seenDocuments(&memory);
     openedDocUrls.reserve(sortedViews.size());
 
-    const auto collectDoc = [&openedDocUrls, &seenDocuments, &allDocuments](KTextEditor::Document *doc) {
+    const auto collectDoc = [&openedDocUrls, &seenDocuments, &allDocuments, &strings](KTextEditor::Document *doc) {
         // we don't want nullptr (bug 497150) or any duplicates, beside for untitled documents
         if (!doc || !seenDocuments.insert(doc).second) {
             return;
@@ -127,12 +132,14 @@ void KateQuickOpenModel::refresh(KateMainWindow *mainWindow)
         if (!doc->url().isEmpty()) {
             auto path = doc->url().toString(QUrl::NormalizePathSegments | QUrl::PreferLocalFile);
             openedDocUrls.insert(path);
-            allDocuments.push_back({QFileInfo(path).fileName(), path, doc, -1});
+            strings.push_back(QFileInfo(path).fileName());
+            allDocuments.push_back({strings.back(), path, doc, -1});
             return;
         }
 
         // untitled document
-        allDocuments.push_back({doc->documentName(), QString(), doc, -1});
+        strings.push_back(doc->documentName());
+        allDocuments.push_back({strings.back(), QString(), doc, -1});
     };
 
     for (auto *view : sortedViews) {
@@ -143,7 +150,7 @@ void KateQuickOpenModel::refresh(KateMainWindow *mainWindow)
         collectDoc(doc);
     }
 
-    for (const auto &filePath : projectDocs) {
+    for (const QString &filePath : projectDocs) {
         // No duplicates
         if (openedDocUrls.count(filePath) != 0) {
             continue;
@@ -151,18 +158,20 @@ void KateQuickOpenModel::refresh(KateMainWindow *mainWindow)
 
         // QFileInfo is too expensive just for fileName computation
         const int slashIndex = filePath.lastIndexOf(QLatin1Char('/'));
-        QString fileName = filePath.mid(slashIndex + 1);
-        allDocuments.push_back({std::move(fileName), filePath, nullptr, -1});
+        QStringView fileName = QStringView(filePath).mid(slashIndex + 1);
+        allDocuments.push_back({fileName, filePath, nullptr, -1});
     }
 
     // Add projects to the docunents list, and the filepath is their base directory
     if (projects.count() > 1) {
-        for (const auto &project : projects.asKeyValueRange()) {
-            allDocuments.push_back({i18n("Project: %1", project.second.toString()), project.first, nullptr, -1});
+        for (const auto &[projectBaseDir, projectName] : projects.asKeyValueRange()) {
+            strings.push_back(i18n("Project: %1", projectName));
+            allDocuments.push_back({strings.back(), projectBaseDir, nullptr, -1});
         }
     }
 
     beginResetModel();
     m_modelEntries = std::move(allDocuments);
+    m_strings = std::move(strings);
     endResetModel();
 }
